{----------------------------------------------------------
 TScrollText: Version 1.0  14 October, 1996.
 by Darryl West (dwest@dot.net.au)
 ----------------------------------------------------------
 Renamed to TxMarquee, modified by Kuan-Ta Chen
 Xshadow@cs.nthu.edu.tw
 ----------------------------------------------------------}
unit ScrollText;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, XAmazing;

type
  TTextStyle = (tsNormal, tsRaised, tsLowered, tsShaddow);
  TScrollDirection = (sdRightToLeft, sdLeftToRight, sdTopToBottom, sdBottomToTop);
  TAlignment = (taCenter, taLeftJustify, taRightJustify);
  TOwnerDrawEvent = procedure (Sender: TObject; Canvas: TCanvas) of object;

  TCustomScrollText = class(TGraphicControl)
  private
    FAlignment: TAlignment;
    FTextStyle: TTextStyle;
    FScrollDirection: TScrollDirection;
    FTimer: TTimer;
    FItems: TStringList;
    FColor: TColor;
    FContinuous: Boolean;
    FEnabled: Boolean;
    FFont: TFont;
    FOnBegin, FOnStep, FOnEnd: TNotifyEvent;
    FOnBeforeDraw, FOnAfterDraw: TOwnerDrawEvent;
    FSteps, FTickSpeed, FSpeed, FCurrentTick, FDepth, LineHi, FCurrentStep, FTextWidth,
      FTextHeight, XPos, YPos: Integer;
    FXAmazing: TXAmazing;
    
    FPolygonNum: Integer;
    FVertexNum: Integer;
    FAllSameColor: Boolean;
    FPolygonColor: TColor; // useful when AllSameColor is True
    FAutoChangeColor: Boolean; // useless when AllSameColor is True
    FChangeColorStep: Byte;
    FDarkColor: Byte;
    FBrightColor: Byte;
    FStepNum: Integer;
    
    procedure SetAlignment(Value: TAlignment);
    procedure SetContinuous(Value: Boolean);
    procedure SetItems(Value: TStringList);
    procedure DataChanged;
    procedure SetTextStyle(Value: TTextStyle);
    procedure SetDirection(Value: TScrollDirection);
    procedure SetSteps(Value: Integer);
    procedure SetTickSpeed(Value: Integer);
    procedure SetColor(Value: TColor);
    procedure SetFont(Value: TFont);
    procedure SetDepth(Value: Integer);
    procedure SetEnabled(Value: Boolean);
    procedure SetSizeParams;
    procedure FontChanged(Sender: TObject);
    procedure DoTextOut(ACanvas: TCanvas; X, Y: Integer; AText: string);
    
    procedure SetPolygonNum(Value: Integer);
    procedure SetVertexNum(Value: Integer);
    procedure SetDarkColor(Value: Byte);
    procedure SetBrightColor(Value: Byte);
    procedure SetStepNum(Value: Integer);
    procedure SetPolygonColor(Value: TColor);
    procedure SetAllSameColor(Value: Boolean);
    procedure SetAutoChangeColor(Value: Boolean);
    procedure SetChangeColorStep(Value: Byte);
  protected
    procedure Paint; override;
    procedure TimerTick(Sender: TObject);
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Depth: Integer read FDepth write SetDepth default 1;
    property ScrollDirection: TScrollDirection read FScrollDirection
      write SetDirection default sdRightToLeft;
    property Items: TStringList read FItems write SetItems;
    property OnBegin: TNotifyEvent read FOnBegin write FOnBegin;
    property OnStep: TNotifyEvent read FOnStep write FOnStep;
    property OnEnd: TNotifyEvent read FOnEnd write FOnEnd;
    property OnBeforeDraw: TOwnerDrawEvent read FOnBeforeDraw write FOnBeforeDraw;
    property OnAfterDraw: TOwnerDrawEvent read FOnAfterDraw write FOnAfterDraw;
    
    property DarkColor: Byte read FDarkColor write SetDarkColor;
    property BrightColor: Byte read FBrightColor write SetBrightColor;
    property PolygonNum: Integer read FPolygonNum write SetPolygonNum;
    property VertexNum: Integer read FVertexNum write SetVertexNum;
    property StepNum: Integer read FStepNum write SetStepNum;
    property AllSameColor: Boolean read FAllSameColor write SetAllSameColor;
    property PolygonColor: TColor read FPolygonColor write SetPolygonColor;
    property AutoChangeColor: Boolean read FAutoChangeColor write SetAutoChangeColor;
    property ChangeColorStep: Byte read FChangeColorStep write SetChangeColorStep;
  public
    property CurrentStep: Integer read FCurrentStep;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReverseDirection;
  published
    property TextStyle: TTextStyle read FTextStyle write SetTextStyle default tsNormal;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Steps: Integer read FSteps write SetSteps default 66;
    property Speed: Integer read FSpeed write FSpeed default 4;
    property TickSpeed: Integer read FTickSpeed write SetTickSpeed default 10;
    property Color: TColor read FColor write SetColor default clBtnFace;
    property Continuous: Boolean read FContinuous write SetContinuous;
    property Font: TFont read FFont write SetFont;
  end;
  
  TScrollText = class(TCustomScrollText)
  published
    property Align;
    property Alignment;
    property Color;
    property Depth;
    property Items;
    property TextStyle;
    property ParentShowHint;
    property Font;
    property ScrollDirection;
    property ShowHint;
    property Speed;
    property TickSpeed;
    property Steps;
    property Visible;
    
    property DarkColor: Byte read FDarkColor write SetDarkColor;
    property BrightColor: Byte read FBrightColor write SetBrightColor;
    property PolygonNum: Integer read FPolygonNum write SetPolygonNum;
    property VertexNum: Integer read FVertexNum write SetVertexNum;
    property StepNum: Integer read FStepNum write SetStepNum;
    property AllSameColor: Boolean read FAllSameColor write SetAllSameColor;
    property PolygonColor: TColor read FPolygonColor write SetPolygonColor;
    property AutoChangeColor: Boolean read FAutoChangeColor write SetAutoChangeColor;
    property ChangeColorStep: Byte read FChangeColorStep write SetChangeColorStep;
    
    property OnBegin;
    property OnStep;
    property OnEnd;
    property OnBeforeDraw;
    property OnAfterDraw;
    
  end;
  
procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Delphi Workshop',[TScrollText]);
end;

constructor TCustomScrollText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  
  ControlStyle := ControlStyle - [csOpaque];
  FItems := TStringList.Create;
  FItems.Add('ScrollText');
  
  Width := 200;
  Height := 20;
  FColor := clBtnFace;
  FSteps := 80;
  FCurrentStep := 0;
  FDepth := 1;
  FContinuous := True;
  FTextStyle := tsNormal;
  FAlignment := taCenter;
  FEnabled := False;
  
  FFont := TFont.Create;
  with FFont do
  begin
    name := 'Arial';
    Size := 10;
    Color := clBlack;
  end;
  FFont.OnChange := FontChanged;
  
  FTickSpeed := 10;
  FSpeed := 5;
  FCurrentTick := 0;
  
  FTimer := TTimer.Create(Self);
  with FTimer do
  begin
    Enabled := False;
    OnTimer := TimerTick;
    Interval := FTickSpeed;
  end;
  
  FXAmazing := TXAmazing.Create;
  with FXAmazing do
  begin
    ;
  end;
  
  ScrollDirection := sdRightToLeft;
  SetDirection(FScrollDirection); { Start scrolling if necessary. }
end;

destructor TCustomScrollText.Destroy;
begin
  FItems.Free;
  FTimer.Free;
  FFont.Free;
  FXAmazing.Free;
  inherited Destroy;
end;

procedure TCustomScrollText.SetItems(Value: TStringList);
begin
  if FItems <> Value then
  begin
    FItems.Assign(Value);
    DataChanged;
  end;
end;

procedure TCustomScrollText.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    FTimer.Enabled := Value;
  end;
end;

procedure TCustomScrollText.DoTextOut(ACanvas: TCanvas; X, Y: Integer; AText: string);
var TextAdjustment: Integer;
begin
  with ACanvas do
  begin
    Font := FFont;
    Brush.Style := bsClear;
    
    if FAlignment = taCenter then
      TextAdjustment := Round((FTextWidth / 2) - (TextWidth(AText) / 2))
    else if FAlignment = taRightJustify then
      TextAdjustment := Round(FTextWidth - TextWidth(AText))
    else TextAdjustment := 0;
    
    case FTextStyle of
      tsRaised:
      begin
        Font.Color := clBtnHighlight;
        TextOut(X - FDepth + TextAdjustment, Y - FDepth, AText);
        Font.Color := clBtnShadow;
        TextOut(X + FDepth + TextAdjustment, Y + FDepth, AText);
      end;
      tsLowered:
      begin
        Font.Color := clBtnShadow;
        TextOut(X - FDepth + TextAdjustment, Y - FDepth, AText);
        Font.Color := clBtnHighlight;
        TextOut(X + FDepth + TextAdjustment, Y + FDepth, AText);
      end;
      tsShaddow:
      begin
        Font.Color := clBtnShadow;
        TextOut(X + FDepth + TextAdjustment, Y + FDepth, AText);
      end;
    end;
    Font.Color := FFont.Color;
    TextOut(X + TextAdjustment, Y, AText);
  end;
end;

procedure TCustomScrollText.Paint;
var
  Bmp                    : TBitmap;
  StartXPos, StartYPos, I: Integer;
  PercentDone            : Double; 
begin
  SetSizeParams;
  Bmp := TBitmap.Create;
  Bmp.Width := Width;
  Bmp.Height := Height;
  
  with Bmp.Canvas do
  begin
    Font := FFont;
    Brush.Color := FColor;
    Brush.Style := bsSolid;
    FillRect(ClipRect);
  end;
  
  FXAmazing.Canvas := Bmp.Canvas;
  
  if FTextWidth >= Width then XPos := 0 else XPos := (Width - FTextWidth) div 2;
  if FTextHeight >= Height then YPos := 0 else YPos := (Height - FTextHeight) div 2;
  
  PercentDone := FCurrentStep / FSteps;
  
  case FScrollDirection of
    sdRightToLeft:
    begin
      StartYPos := YPos;
      StartXPos := Round((FTextWidth + Width) * (1 - PercentDone)) - FTextWidth;
    end;
    sdLeftToRight:
    begin
      StartYPos := YPos;
      StartXPos := Round((FTextWidth + Width) * PercentDone) - FTextWidth;
    end;
    sdBottomToTop:
    begin
      StartXPos := XPos;
      StartYPos := Round((FTextHeight + Height) * (1 - PercentDone)) - FTextHeight;
    end;
    sdTopToBottom:
    begin
      StartXPos := XPos;
      StartYPos := Round((FTextHeight + Height) * PercentDone) - FTextHeight;
    end;
  else
  begin { static }
    StartXPos := XPos;
    StartYPos := YPos;
  end
end;

if Assigned(FOnBeforeDraw) then FOnBeforeDraw(Self, Bmp.Canvas);
FXAmazing.Step;

I := 0;
while I < FItems.Count do
begin
  DoTextOut(Bmp.Canvas, StartXPos, StartYPos, FItems.Strings[I]);
  Inc(StartYPos, LineHi);
  Inc(I);
end;

if Assigned(FOnAfterDraw) then FOnAfterDraw(Self, Bmp.Canvas);

Canvas.Draw(0, 0, Bmp);
Bmp.Free;
end;

procedure TCustomScrollText.SetSizeParams;
var S: string;
  I, SWidth: Integer;    
  Metrics  : TTextMetric;
begin
  with Canvas do
  begin
    Font := FFont;
    GetTextMetrics(Handle, Metrics);
    LineHi := Metrics.tmHeight + Metrics.tmInternalLeading;
    if FTextStyle in [tsRaised, tsLowered] then LineHi := LineHi + 2 * FDepth
    else if FTextStyle in [tsShaddow] then LineHi := LineHi + FDepth;
  end;
  FTextWidth := 0;
  I := 0;
  while I < FItems.Count do
  begin
    S := FItems.Strings[I];
    SWidth := Canvas.TextWidth(S);
    if FTextStyle in [tsRaised, tsLowered] then SWidth := SWidth + 2 * FDepth
    else if FTextStyle in [tsShaddow] then SWidth := SWidth + FDepth;
    if FTextWidth < SWidth then FTextWidth := SWidth;
    Inc(I);
  end;
  FTextHeight := LineHi * FItems.Count;
  if FTextWidth >= Width then XPos := 0 else XPos := (Width - FTextWidth) div 2;
  if FTextHeight >= Height then YPos := 0 else YPos := (Height - FTextHeight) div 2;
end;

procedure TCustomScrollText.DataChanged;
begin
  SetSizeParams;
  Invalidate;
end;

procedure TCustomScrollText.SetTextStyle(Value: TTextStyle);
begin
  if FTextStyle <> Value then
  begin
    FTextStyle := Value;
    DataChanged;
  end;
end;

procedure TCustomScrollText.SetDirection(Value: TScrollDirection);
begin
  if FScrollDirection <> Value then FScrollDirection := Value;
end;

procedure TCustomScrollText.SetContinuous(Value: Boolean);
begin
  if FContinuous <> Value then FContinuous := Value;
end;

procedure TCustomScrollText.SetSteps(Value: Integer);
begin
  if FSteps <> Value then
  begin
    FSteps := Value;
    if csDesigning in ComponentState then Invalidate;
  end;
end;

procedure TCustomScrollText.SetTickSpeed(Value: Integer);
begin
  if FTickSpeed <> Value then
  begin
    if Value > 1000 then Value := 1000
    else if Value < 1 then Value := 1;
    FTickSpeed := Value;
    if FTimer <> nil then FTimer.Interval := FTickSpeed;
  end;
end;

procedure TCustomScrollText.SetColor(Value: TColor);
begin
  if FColor <> Value then
  begin
    FColor := Value;
    DataChanged;
  end;
end;

procedure TCustomScrollText.FontChanged(Sender: TObject);
begin
  DataChanged;
end;

procedure TCustomScrollText.SetFont(Value: TFont);
begin
  if FFont <> Value then
  begin
    FFont.Assign(Value);
    DataChanged;
  end;
end;

procedure TCustomScrollText.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    DataChanged;
  end;
end;

procedure TCustomScrollText.SetDepth(Value: Integer);
begin
  if FDepth <> Value then
  begin
    FDepth := Value;
    DataChanged;
  end;
end;

procedure TCustomScrollText.ReverseDirection;
begin
  FCurrentStep := FSteps - FCurrentStep;
  case FScrollDirection of
    sdLeftToRight: FScrollDirection := sdRightToLeft;
    sdRightToLeft: FScrollDirection := sdLeftToRight;
    sdTopToBottom: FScrollDirection := sdBottomToTop;
    sdBottomToTop: FScrollDirection := sdTopToBottom;
  end;
end;

procedure TCustomScrollText.TimerTick(Sender: TObject);
begin
  if not FTimer.Enabled then Exit;
  if (FCurrentStep = 0) and Assigned(FOnBegin) then FOnBegin(Self);
  
  Inc(FCurrentTick);
  if FCurrentTick >= FSpeed then
  begin
    FCurrentTick := 0;
    Inc(FCurrentStep);
  end;
  
  Paint;
  if Assigned(FOnStep) then FOnStep(Self);
  if FCurrentStep > FSteps then
  begin
    FTimer.Enabled := False;
    if Assigned(FOnEnd) then FOnEnd(Self);
    FCurrentStep := 0;
    if FContinuous then FTimer.Enabled := True;
  end;
end;

procedure TCustomScrollText.SetPolygonNum(Value: Integer);
begin
  if FPolygonNum <> Value then
  begin
    FPolygonNum := Value;
    FXAmazing.PolygonNum := FPolygonNum;
  end;
end;

procedure TCustomScrollText.SetVertexNum(Value: Integer);
begin
  if FVertexNum <> Value then
  begin
    FVertexNum := Value;
    FXAmazing.VertexNum := FVertexNum;
  end;
end;

procedure TCustomScrollText.SetDarkColor(Value: Byte);
begin
  if FDarkColor <> Value then
  begin
    FDarkColor := Value;
    FXAmazing.DarkColor := FDarkColor;
  end;
end;

procedure TCustomScrollText.SetBrightColor(Value: Byte);
begin
  if FBrightColor <> Value then
  begin
    FBrightColor := Value;
    FXAmazing.BrightColor := FBrightColor;
  end;
end;

procedure TCustomScrollText.SetStepNum(Value: Integer);
begin
  if FStepNum <> Value then
  begin
    FStepNum := Value;
    FXAmazing.StepNum := FStepNum;
  end;
end;

procedure TCustomScrollText.SetPolygonColor(Value: TColor);
begin
  if FPolygonColor <> Value then
  begin
    FPolygonColor := Value;
    FXAmazing.PolygonColor := FPolygonColor;
  end;
end;

procedure TCustomScrollText.SetAllSameColor(Value: Boolean);
begin
  if FAllSameColor <> Value then
  begin
    FAllSameColor := Value;
    FXAmazing.AllSameColor := FAllSameColor;
  end;
end;

procedure TCustomScrollText.SetAutoChangeColor(Value: Boolean);
begin
  if FAutoChangeColor <> Value then
  begin
    FAutoChangeColor := Value;
    FXAmazing.AutoChangeColor := FAutoChangeColor;
  end;
end;

procedure TCustomScrollText.SetChangeColorStep(Value: Byte);
begin
  if FChangeColorStep <> Value then
  begin
    FChangeColorStep := Value;
    FXAmazing.ChangeColorStep := FChangeColorStep;
  end;
end;



end.

