unit xamazing;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls;

const
  MaxPolygonNum = 20; 
  MaxVertexNum  = 20; 
  MaxStepTick   = 5;  
  MaxSpeed      = 500;
  
type
  TChangeColor = (ccRedInc, ccRedDec, ccGreenInc, ccGreenDec, ccBlueInc, ccBlueDec);
  
  TXAmazing = class(TObject)
  private
    { Private declarations }
    FPolygonNum: Integer;
    
    FVertexNum: Integer;
    
    FVertexs: array[0..MaxPolygonNum, 0..MaxVertexNum] of TPoint;
    FStepTick: array[0..MaxPolygonNum, 0..MaxVertexNum] of TPoint;
    FCurrentTick: array[0..MaxPolygonNum, 0..MaxVertexNum] of TPoint;
    FDirectionX: array[0..MaxPolygonNum, 0..MaxVertexNum] of Boolean;
    FDirectionY: array[0..MaxPolygonNum, 0..MaxVertexNum] of Boolean;
    
    { True: Right/Bottom,  False: Left/Top }
    
    FAllSameColor: Boolean;
    FPolygonColor: TColor; // useful when AllSameColor is True
    FPolygonColors: array[0..MaxPolygonNum] of TColor;
    
    FAutoChangeColor: Boolean; // useless when AllSameColor is True
    FChangeColorStep: Byte;
    FColorChanging: array[0..MaxPolygonNum] of TChangeColor;
    FDarkColor: Byte;
    FBrightColor: Byte;
    FStepNum: Integer;
    
    FTimer: TTimer;
    FSpeed: Integer;
    FEnabled: Boolean;
    
    FCanvas: TCanvas;
    FWidth, FHeight: Integer;
    
    FNeedClear: Boolean;
    FBackGroundColor: TColor;
    
    procedure FOnTimer(Sender: TObject);
    procedure SetEnabled(Value: Boolean);
    procedure SetCanvas(Value: TCanvas);
    procedure SetPolygonNum(Value: Integer);
    procedure SetVertexNum(Value: Integer);
    procedure SetSpeed(Value: Integer);
    procedure SetDarkColor(Value: Byte);
    procedure SetBrightColor(Value: Byte);
    
    procedure InitialPolygon;
    procedure DrawPolygon;
    procedure StepPolygon;
  public
    constructor Create;
    destructor Destroy; override;
    
    procedure Step;
    
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Speed: Integer read FSpeed write SetSpeed;
    property Canvas: TCanvas read FCanvas write SetCanvas;
    property NeedClear: Boolean read FNeedClear write FNeedClear;
    property BackGroundColor: TColor read FBackGroundColor write FBackGroundColor;
    property DarkColor: Byte read FDarkColor write SetDarkColor;
    property BrightColor: Byte read FBrightColor write SetBrightColor;
    
    property PolygonNum: Integer read FPolygonNum write SetPolygonNum;
    property VertexNum: Integer read FVertexNum write SetVertexNum;
    property StepNum: Integer read FStepNum write FStepNum default 1;
    property AllSameColor: Boolean read FAllSameColor write FAllSameColor;
    property PolygonColor: TColor read FPolygonColor write FPolygonColor;
    property AutoChangeColor: Boolean read FAutoChangeColor write FAutoChangeColor;
    property ChangeColorStep: Byte read FChangeColorStep write FChangeColorStep;
  end;
  
implementation

constructor TXAmazing.Create;
begin
  inherited Create;
  
  FEnabled := False;
  FPolygonNum := 4;
  FVertexNum := 4;
  FAllSameColor := False;
  FPolygonColor := clRed;
  FAutoChangeColor := True;
  FChangeColorStep := 1;
  FNeedClear := False;
  FBackGroundColor := clBlack;
  FDarkColor := 128;
  FBrightColor := 255;
  FSpeed := 100;
  FStepNum := 1;
end;

destructor TXAmazing.Destroy;
begin
  FTimer.Free;
  
  inherited Destroy;
end;

procedure TXAmazing.FOnTimer(Sender: TObject);
begin
  StepPolygon;
  DrawPolygon;
end;

procedure TXAmazing.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    if FEnabled then
    begin
      if not Assigned(FCanvas) then
      begin
        FEnabled := False;
        Exit;
      end;
      if not Assigned(FTimer) then
      begin
        FTimer := TTimer.Create(Application);
        FTimer.Interval := Round(1000 / FSpeed);
        FTimer.OnTimer := FOnTimer;
      end;
    end;
    if Assigned(FTimer) then FTimer.Enabled := Value;
  end;
end;

procedure TXAmazing.SetCanvas(Value: TCanvas);
begin
  if FCanvas <> Value then
  begin
    FCanvas := Value;
    FWidth := FCanvas.ClipRect.Right;
    FHeight := FCanvas.ClipRect.Bottom;
    InitialPolygon;
  end;
end;

procedure TXAmazing.SetPolygonNum(Value: Integer);
begin
  if (FPolygonNum <> Value) and (Value >= 0) and (Value <= MaxPolygonNum) then
  begin
    FPolygonNum := Value;
    InitialPolygon;
  end;
end;

procedure TXAmazing.SetVertexNum(Value: Integer);
begin
  if (FVertexNum <> Value) and (Value >= 2) and (Value <= MaxVertexNum) then
  begin
    FVertexNum := Value;
    InitialPolygon;
  end;
end;

procedure TXAmazing.SetSpeed(Value: Integer);
begin
  if (FSpeed <> Value) and (Value > 0) and (Value <= MaxSpeed) then
  begin
    FSpeed := Value;
    FTimer.Interval := Round(1000 / FSpeed);
  end;
end;

procedure TXAmazing.SetBrightColor(Value: Byte);
begin
  if (FBrightColor <> Value) and (Value > FDarkColor) and (Value <= 255) then
    FBrightColor := Value;
end;

procedure TXAmazing.SetDarkColor(Value: Byte);
begin
  if (FDarkColor <> Value) and (Value < FBrightColor) and (Value >= 0) then
    FDarkColor := Value;
end;

procedure TXAmazing.InitialPolygon;
var
  P, V: Integer;
begin
  Randomize;
  for P := 0 to FPolygonNum - 1 do
  begin
    if FAllSameColor then
      FPolygonColors[P] := FPolygonColor
    else
    begin
      FPolygonColors[P] := RGB(Random(FBrightColor - FDarkColor) + FDarkColor, Random(FBrightColor - FDarkColor) + FDarkColor, Random(FBrightColor - FDarkColor) + FDarkColor);
    end;
    FColorChanging[P] := TChangeColor(Random(6));
    
    for V := 0 to VertexNum - 1 do
    begin
      FVertexs[P, V] := Point(Random(FWidth), Random(FHeight));
      FStepTick[P, V] := Point(Random(MaxStepTick - 1) + 1, Random(MaxStepTick - 1) + 1);
      FCurrentTick[P, V] := Point(0, 0);
      
      if Random(2) = 1 then
        FDirectionX[P, V] := True
      else
        FDirectionX[P, V] := False;
      
      if Random(2) = 1 then
        FDirectionY[P, V] := True
      else
        FDirectionY[P, V] := False;
    end;
  end;
end;

procedure TXAmazing.StepPolygon;
var
  P, V            : Integer;     
  R, G, B         : Byte;        
  Finished        : Boolean;     
  OldColorChanging: TChangeColor;
begin
  for P := 0 to FPolygonNum - 1 do
  begin
    for V := 0 to VertexNum - 1 do
    begin
      Inc(FCurrentTick[P, V].X);
      Inc(FCurrentTick[P, V].Y);
      
      if FCurrentTick[P, V].X = FStepTick[P, V].X then
      begin
        if FDirectionX[P, V] then Inc(FVertexs[P, V].X, FStepNum) else Dec(FVertexs[P, V].X, FStepNum);
        if (FVertexs[P, V].X <= 0) or (FVertexs[P, V].X >= FWidth - 1) then
          FDirectionX[P, V] := not FDirectionX[P, V];
        FCurrentTick[P, V].X := 0;
      end;
      
      if FCurrentTick[P, V].Y = FStepTick[P, V].Y then
      begin
        if FDirectionY[P, V] then Inc(FVertexs[P, V].Y, FStepNum) else Dec(FVertexs[P, V].Y, FStepNum);
        if (FVertexs[P, V].Y <= 0) or (FVertexs[P, V].Y >= FHeight - 1) then
          FDirectionY[P, V] := not FDirectionY[P, V];
        FCurrentTick[P, V].Y := 0;
      end;
    end;
    
    if FAutoChangeColor and (not FAllSameColor) then
    begin
      R := GetRValue(FPolygonColors[P]);
      G := GetGValue(FPolygonColors[P]);
      B := GetBValue(FPolygonColors[P]);
      
      Finished := False;
      case FColorChanging[P] of
        ccRedInc:
        begin
          Inc(R, FChangeColorStep);
          if R >= FBrightColor then
          begin
            Finished := True;
            Dec(R, FChangeColorStep);
          end;
        end;
        ccRedDec:
        begin
          Dec(R, FChangeColorStep);
          if R <= FDarkColor then
          begin
            Finished := True;
            Inc(R, FChangeColorStep);
          end;
        end;
        ccGreenInc:
        begin
          Inc(G, FChangeColorStep);
          if G >= FBrightColor then
          begin
            Finished := True;
            Dec(G, FChangeColorStep);
          end;
        end;
        ccGreenDec:
        begin
          Dec(G, FChangeColorStep);
          if G <= FDarkColor then
          begin
            Finished := True;
            Inc(G, FChangeColorStep);
          end;
        end;
        ccBlueInc:
        begin
          Inc(B, FChangeColorStep);
          if B >= FBrightColor then
          begin
            Finished := True;
            Dec(B, FChangeColorStep);
          end;
        end;
        ccBlueDec:
        begin
          Dec(B, FChangeColorStep);
          if B <= FDarkColor then
          begin
            Finished := True;
            Inc(B, FChangeColorStep);
          end;
        end;
      end;
      
      if Finished then
      begin
        OldColorChanging := FColorChanging[P];
        repeat
          FColorChanging[P] := TChangeColor(Random(6));
        until FColorChanging[P] <> OldColorChanging;
      end;
      
      FPolygonColors[P] := RGB(R, G, B);
    end;
  end;
end;

procedure TXAmazing.DrawPolygon;
var
  P: Integer;
begin
  if FPolygonNum = 0 then Exit;
  with FCanvas do
  begin
    if NeedClear then
    begin
      Brush.Style := bsSolid;
      Brush.Color := FBackGroundColor;
      FillRect(ClipRect);
    end;
    Font.Color := clWhite;

{$IFDEF DEBUG}
    TextOut(0, 0, Format('%d %d %d',[GetRValue(FPolygonColors[0]), GetGValue(FPolygonColors[0]), GetBValue(FPolygonColors[0])]));
{$ENDIF}

    Brush.Style := bsClear;
    for P := 0 to FPolygonNum - 1 do
    begin
      Pen.Color := FPolygonColors[P];
      POLYGON(Slice(FVertexs[P], FVertexNum));
    end;
  end;
end;

procedure TXAmazing.Step;
begin
  StepPolygon;
  DrawPolygon;
end;

end.

