{----------------------------------------------------------
 TScrollText: Version 1.0  14 October, 1996.
 by Darryl West (dwest@dot.net.au)
 ----------------------------------------------------------
 Renamed to TxMarquee, modified by Kuan-Ta Chen
 kuan@vclxx.com        
 ----------------------------------------------------------}
unit xmarquee;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, ExtCtrls, XAmazing;

type
  TTextStyle = (tsNormal, tsRaised, tsLowered, tsShaddow);
  TScrollDirection = (sdRightToLeft, sdLeftToRight, sdTopToBottom, sdBottomToTop);
  TAlignment = (taCenter, taLeftJustify, taRightJustify);
  TOwnerDrawEvent = procedure (Sender: TObject; Canvas: TCanvas) of object;
  
  TxMarquee = class(TGraphicControl)
  private
    FAlignment: TAlignment;
    FTextStyle: TTextStyle;
    FScrollDirection: TScrollDirection;
    
    FTimer: TTimer;
    FLines: TStringList;
    FColor: TColor;
    FContinuous: Boolean;
    FEnabled: Boolean;
    FFont: TFont;
    
    FOnBegin, FOnStep, FOnEnd: TNotifyEvent;
    FOnBeforeDraw, FOnAfterDraw: TOwnerDrawEvent;
    
    FSteps, FTickInterval, FDepth, LineHi, FCurrentStep: Integer;
    FTextWidth, FTextHeight, XPos, YPos: Integer;
    FXAmazing: TXAmazing;
    
    FPolygonNum, FPolygonVertexNum: Integer;
    FAllSameColor: Boolean;
    
    FPolygonColor: TColor; // useful when AllSameColor is True
    FAutoChangeColor: Boolean; // useless when AllSameColor is True
    
    FChangeColorStep: Byte;
    FDarkColor: Byte;
    FBrightColor: Byte;
    FPolygonPolygonStepNum: Integer;
    
    procedure SetAlignment(Value: TAlignment);
    procedure SetLines(Value: TStringList);
    procedure DataChanged;
    procedure SetTextStyle(Value: TTextStyle);
    procedure SetSteps(Value: Integer);
    procedure SetTickInterval(Value: Integer);
    procedure SetColor(Value: TColor);
    procedure SetFont(Value: TFont);
    procedure SetDepth(Value: Integer);
    procedure SetEnabled(Value: Boolean);
    procedure SetSizeParams;
    procedure FontChanged(Sender: TObject);
    procedure DoTextOut(ACanvas: TCanvas; X, Y: Integer; AText: string);
    
    procedure SetPolygonNum(Value: Integer);
    procedure SetPolygonVertexNum(Value: Integer);
    procedure SetDarkColor(Value: Byte);
    procedure SetBrightColor(Value: Byte);
    procedure SetPolygonPolygonStepNum(Value: Integer);
    procedure SetPolygonColor(Value: TColor);
    procedure SetAllSameColor(Value: Boolean);
    procedure SetAutoChangeColor(Value: Boolean);
    procedure SetChangeColorStep(Value: Byte);
  protected
    procedure Paint; override;
    procedure TimerTick(Sender: TObject);
    
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property CurrentStep: Integer read FCurrentStep write FCurrentStep;
    procedure ReverseDirection;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property AllSameColor: Boolean read FAllSameColor write SetAllSameColor;
    property AutoChangeColor: Boolean read FAutoChangeColor write SetAutoChangeColor;
    property BrightColor: Byte read FBrightColor write SetBrightColor;
    property ChangeColorStep: Byte read FChangeColorStep write SetChangeColorStep;
    property Color: TColor read FColor write SetColor default clBtnFace;
    property Continuous: Boolean read FContinuous write FContinuous;
    property DarkColor: Byte read FDarkColor write SetDarkColor;
    property Depth: Integer read FDepth write SetDepth default 1;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Font: TFont read FFont write SetFont;
    property Lines: TStringList read FLines write SetLines;
    property PolygonColor: TColor read FPolygonColor write SetPolygonColor;
    property PolygonNum: Integer read FPolygonNum write SetPolygonNum;
    property PolygonPolygonStepNum: Integer read FPolygonPolygonStepNum write SetPolygonPolygonStepNum default 1;
    property PolygonVertexNum: Integer read FPolygonVertexNum write SetPolygonVertexNum;
    property ScrollDirection: TScrollDirection read FScrollDirection write FScrollDirection default sdRightToLeft;
    property Steps: Integer read FSteps write SetSteps default 80;
    property TextStyle: TTextStyle read FTextStyle write SetTextStyle default tsNormal;
    property TickInterval: Integer read FTickInterval write SetTickInterval default 50;

    property Align;
    property ParentColor;    
    property ParentFont;    
    property ParentShowHint;
    property ShowHint;
    property Visible;
    
    property OnBegin: TNotifyEvent read FOnBegin write FOnBegin;
    property OnStep: TNotifyEvent read FOnStep write FOnStep;
    property OnEnd: TNotifyEvent read FOnEnd write FOnEnd;
    property OnBeforeDraw: TOwnerDrawEvent read FOnBeforeDraw write FOnBeforeDraw;
    property OnAfterDraw: TOwnerDrawEvent read FOnAfterDraw write FOnAfterDraw;
  end;
  
procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Delphi Workshop',[TxMarquee]);
end;

constructor TxMarquee.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  
  ControlStyle := ControlStyle - [csOpaque];
  FLines := TStringList.Create;
  FLines.Add('xMarquee');
  
  Width := 200;
  Height := 20;
  FColor := clBtnFace;
  FSteps := 80;
  FPolygonPolygonStepNum := 1;
  FCurrentStep := 0;
  FDepth := 1;
  FContinuous := True;
  FTextStyle := tsNormal;
  FAlignment := taCenter;
  FEnabled := False;

  FFont := TFont.Create;
  with FFont do
  begin
    name := 'Arial';
    Size := 10;
    Color := clBlack;
  end;
  FFont.OnChange := FontChanged;

  FTickInterval := 50;
  
  FTimer := TTimer.Create(Self);
  with FTimer do
  begin
    Enabled := False;
    OnTimer := TimerTick;
    Interval := FTickInterval;
  end;
  
  FXAmazing := TXAmazing.Create;
  with FXAmazing do
  begin
    ;
  end;
  
  ScrollDirection := sdRightToLeft;
end;

destructor TxMarquee.Destroy;
begin
  FLines.Free;
  FTimer.Free;
  FFont.Free;
  FXAmazing.Free;
  
  inherited Destroy;
end;

procedure TxMarquee.SetLines(Value: TStringList);
begin
  if FLines <> Value then
  begin
    FLines.Assign(Value);
    DataChanged;
  end;
end;

procedure TxMarquee.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    FTimer.Enabled := Value;
  end;
end;

procedure TxMarquee.DoTextOut(ACanvas: TCanvas; X, Y: Integer; AText: string);
var
  TextAdjustment: Integer;
begin
  with ACanvas do
  begin
    Font := FFont;
    Brush.Style := bsClear;
    
    if FAlignment = taCenter then
      TextAdjustment := Round((FTextWidth / 2) - (TextWidth(AText) / 2))
    else if FAlignment = taRightJustify then
      TextAdjustment := Round(FTextWidth - TextWidth(AText))
    else TextAdjustment := 0;
    
    case FTextStyle of
      tsRaised:
      begin
        Font.Color := clBtnHighlight;
        TextOut(X - FDepth + TextAdjustment, Y - FDepth, AText);
        Font.Color := clBtnShadow;
        TextOut(X + FDepth + TextAdjustment, Y + FDepth, AText);
      end;
      
      tsLowered:
      begin
        Font.Color := clBtnShadow;
        TextOut(X - FDepth + TextAdjustment, Y - FDepth, AText);
        Font.Color := clBtnHighlight;
        TextOut(X + FDepth + TextAdjustment, Y + FDepth, AText);
      end;
      
      tsShaddow:
      begin
        Font.Color := clBtnShadow;
        TextOut(X + FDepth + TextAdjustment, Y + FDepth, AText);
      end;
    end;
    
    Font.Color := FFont.Color;
    TextOut(X + TextAdjustment, Y, AText);
  end;
end;

procedure TxMarquee.Paint;
var
  Bmp                    : TBitmap;
  StartXPos, StartYPos, I: Integer;
  PercentDone            : Double; 
begin
  SetSizeParams;

  Bmp := TBitmap.Create;
  Bmp.Width := Width;
  Bmp.Height := Height;

  with Bmp.Canvas do
  begin
    Font := FFont;
    Brush.Color := FColor;
    Brush.Style := bsSolid;
    FillRect(ClipRect);
  end;

  FXAmazing.Canvas := Bmp.Canvas;

  if FTextWidth >= Width then XPos := 0 else XPos := (Width - FTextWidth) div 2;
  if FTextHeight >= Height then YPos := 0 else YPos := (Height - FTextHeight) div 2;
  
  PercentDone := FCurrentStep / FSteps;
  
  case FScrollDirection of
    sdRightToLeft:
    begin
      StartYPos := YPos;
      StartXPos := Round((FTextWidth + Width) * (1 - PercentDone)) - FTextWidth;
    end;
    
    sdLeftToRight:
    begin
      StartYPos := YPos;
      StartXPos := Round((FTextWidth + Width) * PercentDone) - FTextWidth;
    end;
    
    sdBottomToTop:
    begin
      StartXPos := XPos;
      StartYPos := Round((FTextHeight + Height) * (1 - PercentDone)) - FTextHeight;
    end;
    
    sdTopToBottom:
    begin
      StartXPos := XPos;
      StartYPos := Round((FTextHeight + Height) * PercentDone) - FTextHeight;
    end;
  else
  begin { static }
    StartXPos := XPos;
    StartYPos := YPos;
  end
end;

if Assigned(FOnBeforeDraw) then FOnBeforeDraw(Self, Bmp.Canvas);
FXAmazing.Step;

I := 0;
while I < FLines.Count do
begin
  DoTextOut(Bmp.Canvas, StartXPos, StartYPos, FLines.Strings[I]);
  Inc(StartYPos, LineHi);
  Inc(I);
end;

if Assigned(FOnAfterDraw) then FOnAfterDraw(Self, Bmp.Canvas);

Canvas.Draw(0, 0, Bmp);
Bmp.Free;
end;

procedure TxMarquee.SetSizeParams;
var
  S        : string;     
  I, SWidth: Integer;    
  Metrics  : TTextMetric;
begin
  with Canvas do
  begin
    Font := FFont;
    GetTextMetrics(Handle, Metrics);
    LineHi := Metrics.tmHeight + Metrics.tmInternalLeading;
    if FTextStyle in [tsRaised, tsLowered] then
      LineHi := LineHi + 2 * FDepth
    else if FTextStyle in [tsShaddow] then
      LineHi := LineHi + FDepth;
  end;
  
  FTextWidth := 0;
  I := 0;
  while I < FLines.Count do
  begin
    S := FLines.Strings[I];
    SWidth := Canvas.TextWidth(S);
    if FTextStyle in [tsRaised, tsLowered] then
      SWidth := SWidth + 2 * FDepth
    else if FTextStyle in [tsShaddow] then
      SWidth := SWidth + FDepth;
    
    if FTextWidth < SWidth then FTextWidth := SWidth;
    Inc(I);
  end;
  
  FTextHeight := LineHi * FLines.Count;
  if FTextWidth >= Width then XPos := 0 else XPos := (Width - FTextWidth) div 2;
  if FTextHeight >= Height then YPos := 0 else YPos := (Height - FTextHeight) div 2;
end;

procedure TxMarquee.DataChanged;
begin
  SetSizeParams;
  Invalidate;
end;

procedure TxMarquee.SetTextStyle(Value: TTextStyle);
begin
  if FTextStyle <> Value then
  begin
    FTextStyle := Value;
    DataChanged;
  end;
end;

procedure TxMarquee.SetSteps(Value: Integer);
begin
  if FSteps <> Value then
  begin
    FSteps := Value;
    if csDesigning in ComponentState then Invalidate;
  end;
end;

procedure TxMarquee.SetTickInterval(Value: Integer);
begin
  if FTickInterval <> Value then
  begin
    if Value > 1000 then Value := 1000
    else if Value < 1 then Value := 1;
    FTickInterval := Value;
    if FTimer <> nil then FTimer.Interval := FTickInterval;
  end;
end;

procedure TxMarquee.SetColor(Value: TColor);
begin
  if FColor <> Value then
  begin
    FColor := Value;
    DataChanged;
  end;
end;

procedure TxMarquee.FontChanged(Sender: TObject);
begin
  DataChanged;
end;

procedure TxMarquee.SetFont(Value: TFont);
begin
  if FFont <> Value then
  begin
    FFont.Assign(Value);
    DataChanged;
  end;
end;

procedure TxMarquee.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    DataChanged;
  end;
end;

procedure TxMarquee.SetDepth(Value: Integer);
begin
  if FDepth <> Value then
  begin
    FDepth := Value;
    DataChanged;
  end;
end;

procedure TxMarquee.ReverseDirection;
begin
  FCurrentStep := FSteps - FCurrentStep;
  
  case FScrollDirection of
    sdLeftToRight: FScrollDirection := sdRightToLeft;
    sdRightToLeft: FScrollDirection := sdLeftToRight;
    sdTopToBottom: FScrollDirection := sdBottomToTop;
    sdBottomToTop: FScrollDirection := sdTopToBottom;
  end;
end;

procedure TxMarquee.TimerTick(Sender: TObject);
begin
  if not FTimer.Enabled then Exit;
  if (FCurrentStep = 0) and Assigned(FOnBegin) then FOnBegin(Self);
  
  Inc(FCurrentStep);

  Paint;

  if Assigned(FOnStep) then FOnStep(Self);

  if FCurrentStep > FSteps then
  begin
    FTimer.Enabled := False;

    if Assigned(FOnEnd) then FOnEnd(Self);

    FCurrentStep := 0;
    if FContinuous then FTimer.Enabled := True;
  end;
end;

procedure TxMarquee.SetPolygonNum(Value: Integer);
begin
  if FPolygonNum <> Value then
  begin
    FPolygonNum := Value;
    FXAmazing.PolygonNum := FPolygonNum;
  end;
end;

procedure TxMarquee.SetPolygonVertexNum(Value: Integer);
begin
  if FPolygonVertexNum <> Value then
  begin
    FPolygonVertexNum := Value;
    FXAmazing.VertexNum := FPolygonVertexNum;
  end;
end;

procedure TxMarquee.SetDarkColor(Value: Byte);
begin
  if FDarkColor <> Value then
  begin
    FDarkColor := Value;
    FXAmazing.DarkColor := FDarkColor;
  end;
end;

procedure TxMarquee.SetBrightColor(Value: Byte);
begin
  if FBrightColor <> Value then
  begin
    FBrightColor := Value;
    FXAmazing.BrightColor := FBrightColor;
  end;
end;

procedure TxMarquee.SetPolygonPolygonStepNum(Value: Integer);
begin
  if FPolygonPolygonStepNum <> Value then
  begin
    FPolygonPolygonStepNum := Value;
    FXAmazing.StepNum := FPolygonPolygonStepNum;
  end;
end;

procedure TxMarquee.SetPolygonColor(Value: TColor);
begin
  if FPolygonColor <> Value then
  begin
    FPolygonColor := Value;
    FXAmazing.PolygonColor := FPolygonColor;
  end;
end;

procedure TxMarquee.SetAllSameColor(Value: Boolean);
begin
  if FAllSameColor <> Value then
  begin
    FAllSameColor := Value;
    FXAmazing.AllSameColor := FAllSameColor;
  end;
end;

procedure TxMarquee.SetAutoChangeColor(Value: Boolean);
begin
  if FAutoChangeColor <> Value then
  begin
    FAutoChangeColor := Value;
    FXAmazing.AutoChangeColor := FAutoChangeColor;
  end;
end;

procedure TxMarquee.SetChangeColorStep(Value: Byte);
begin
  if FChangeColorStep <> Value then
  begin
    FChangeColorStep := Value;
    FXAmazing.ChangeColorStep := FChangeColorStep;
  end;
end;

end.

