unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ActnList, ComCtrls, MPlayer;

type
  TMainForm = class(TForm)
    lbSampClasses: TListBox;
    lblBaseClassInfo: TLabel;
    lbBaseClassInfo: TListBox;
    lblClassProperties: TLabel;
    lbPropList: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure lbSampClassesClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses TypInfo;

{$R *.DFM}

function CreateAClass(const AClassName: string): TObject;
var
  C         : TFormClass;
  SomeObject: TObject;
begin
  C := TFormClass(FindClass(AClassName));
  SomeObject := C.Create(nil);
  Result := SomeObject;
end;


procedure GetBaseClassInfo(AClass: TObject; AStrings: TStrings);
var
  ClassTypeInfo: PTypeInfo;
  ClassTypeData: PTypeData;
  EnumName     : string;
begin
  ClassTypeInfo := AClass.ClassInfo;
  ClassTypeData := GetTypeData(ClassTypeInfo);
  with AStrings do
  begin
    Add(Format('Class Name:     %s',[ClassTypeInfo.name]));
    EnumName := GetEnumName(TypeInfo(TTypeKind), Integer(ClassTypeInfo.Kind));
    Add(Format('Kind:           %s',[EnumName]));
    Add(Format('Size:           %d',[AClass.InstanceSize]));
    Add(Format('Defined in:     %s.pas',[ClassTypeData.UnitName]));
    Add(Format('Num Properties: %d',[ClassTypeData.PropCount]));
  end;
end;

procedure GetClassAncestry(AClass: TObject; AStrings: TStrings);
var
  AncestorClass: TClass;
begin
  AncestorClass := AClass.ClassParent;

  AStrings.Add('Class Ancestry');
  while AncestorClass <> nil do
  begin
    AStrings.Add(Format('    %s',[AncestorClass.ClassName]));
    AncestorClass := AncestorClass.ClassParent;
  end;
end;


procedure GetClassProperties(AClass: TObject; AStrings: TStrings);
var
  PropList     : PPropList;
  ClassTypeInfo: PTypeInfo;
  ClassTypeData: PTypeData;
  I            : Integer;
  NumProps     : Integer;
begin
  ClassTypeInfo := AClass.ClassInfo;
  ClassTypeData := GetTypeData(ClassTypeInfo);

  if ClassTypeData.PropCount <> 0 then
  begin
    GetMem(PropList, SizeOf(PPropInfo) * ClassTypeData.PropCount);
    try
      GetPropInfos(AClass.ClassInfo, PropList);
      for I := 0 to ClassTypeData.PropCount - 1 do
        if not (PropList[I]^.PropType^.Kind = tkMethod) then
          AStrings.Add(Format('%s: %s',[PropList[I]^.name, PropList[I]^.PropType^.name]));

      NumProps := GetPropList(AClass.ClassInfo,[tkMethod], PropList);
      if NumProps <> 0 then
      begin
        AStrings.Add('==== Events ====');
      end;
      for I := 0 to NumProps - 1 do
        AStrings.Add(Format('%s: %s',[PropList[I]^.name, PropList[I]^.PropType^.name]));

    finally
      FreeMem(PropList, SizeOf(PPropInfo) * ClassTypeData.PropCount);
    end;
  end;

end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  lbSampClasses.Items.Add('TActionList');
  lbSampClasses.Items.Add('TButton');
  lbSampClasses.Items.Add('TListBox');
  lbSampClasses.Items.Add('TLabel');
  lbSampClasses.Items.Add('TTreeView');
  lbSampClasses.Items.Add('TColorDialog');
  lbSampClasses.Items.Add('TMediaPlayer');
end;

procedure TMainForm.lbSampClassesClick(Sender: TObject);
var
  SomeComp: TObject;
begin
  lbBaseClassInfo.Items.Clear;
  lbPropList.Items.Clear;

  SomeComp := CreateAClass(lbSampClasses.Items[lbSampClasses.ItemIndex]);
  try
    GetBaseClassInfo(SomeComp, lbBaseClassInfo.Items);
    GetClassAncestry(SomeComp, lbBaseClassInfo.Items);
    GetClassProperties(SomeComp, lbPropList.Items);
  finally
    SomeComp.Free;
  end;
end;

initialization
begin
  RegisterClasses([TActionList, TButton, TListBox, TLabel, TTreeView,
    TColorDialog, TMediaPlayer]);
end;

end.

